% //////////////////////////////////////////////////////////////////// %
%           Surprise Linear Model - Copyright (C) 2004-2007            %
% by the University of Southern California (USC) and the iLab at USC.  %
% See http://iLab.usc.edu for information about this project.          %
% //////////////////////////////////////////////////////////////////// %
% This file is part of the iLab Neuromorphic Vision Toolkit            %
%                                                                      %
% The Surprise Linear Model is free software; you can                  %
% redistribute it and/or modify it under the terms of the GNU General  %
% Public License as published by the Free Software Foundation; either  %
% version 2 of the License, or (at your option) any later version.     %
%                                                                      %
% The Surprise Linear Model is distributed in the hope                 %
% that it will be useful, but WITHOUT ANY WARRANTY; without even the   %
% implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      %
% PURPOSE.  See the GNU General Public License for more details.       %
%                                                                      %
% You should have received a copy of the GNU General Public License    %
% along with the iBaysian Surprise Matlab Toolkit; if not, write       %
% to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   %
% Boston, MA 02111-1307 USA.                                           %
% //////////////////////////////////////////////////////////////////// %
%
% Primary maintainer for this file: T. Nathan Mundhenk <mundhenk@usc.edu>
% $Revision: 55 $ 
% $Id$
% $HeadURL: https://surprise-mltk.svn.sourceforge.net/svnroot/surprise-mltk/source/surprise_toolkit/example_graph.m $

% X = LINEAR_MODEL_BUILD_PROCESS_RSVP(CONF)
% Use this function to build a new process_RSVP.pl script given the
% paramers in CONF which is an object

function conf = linear_model_build_process_RSVP(conf,type)

dprint(['Making process_RSVP.pl file - ',conf.procRSVP]);

conf.body.timeStamp = clock;

header = print_header(conf,type);
footer = print_footer(conf);

if strcmp(conf.surprise.useStandard,'yes')
    [body,conf] = print_body_standard(conf,type);
else
    % future use %
end

[rsvpfile, rsvpmsg] = fopen(conf.procRSVP,'wt');
fcount              = fprintf(rsvpfile, header);
fcount              = fprintf(rsvpfile, body);
fcount              = fprintf(rsvpfile, footer);
fclose(rsvpfile);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function header = print_header(conf,type)

header = [ ...    
'#!/usr/bin/perl \n' ...
'\n' ...
'# Auto generated by linear_classify_build_process_rsvp.m \n' ...
'# Created: ' date ' : ' ...
num2str(conf.body.timeStamp(1,4)) '.' num2str(conf.body.timeStamp(1,5)) '.' num2str(conf.body.timeStamp(1,6)) '\n'...
'# Do not edit this file directly \n' ...
'\n' ...
'$condition \t= \"' conf.condString '\"; \n' ...
'$fdur \t\t= \"' conf.surprise.duration '\"; \n' ...
'$redir \t\t= \">/dev/null 2>&1\"; \n' ...
'\n' ...
'foreach $d (@ARGV) \n' ...
'{\n' ...
'\t' '@x = split(/\\//, $d); \n' ... 
'\t' '$fbase = pop(@x); \n'];
if(strcmp(type,'AB'))
    header = [ header ... 
    '\t' '@p = split(/\\_/, ${fbase}); \n' ... 
    '\t' '$fp3 = pop(@p); \n' ...
    '\t' '$fp2 = pop(@p); \n' ...
    '\t' '$fp1 = pop(@p); \n' ...
    ];
end

header = [ header ...
'\n' ...
'\t' '# Clean each file before running since ezvision will append \n' ...
'\t' '$command = \"rm -f $d/chan*txt\"; \n' ...
'\t' 'system($command); \n' ...
'\n' ...
 '\t' '# For each stim sequence call ezvision with this way \n' ...
];


% Hard files have a slightly different header due to file naming
if(strcmp(type,'hard'))
    header = [header ...
        '\n' ...
        '\t' '$filep = \"stim_#\"; \n' ...
        '\n' ...
    ];
end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function footer = print_footer(conf)

if strcmp(conf.surprise.logCommand,'yes')
    %clear out the logdir for use
    command = ['rm -f ' conf.baseDir 'log/process_rsvp.log.txt'];
    fprintf('\t\tclearing log file for RSVP: %s\n',command);
    [status,result] = unix(command);
    
    footer = [ ...
    '\n' ...
    '\t' '# Make a record of each command called, lock the log while doing this \n' ...
    '\t' 'open(LOGFILE, \">>' conf.baseDir 'log/process_rsvp.log.txt\"); \n' ...
    '\t' 'flock LOGFILE, 2; \n' ...
    '\t' 'print(LOGFILE \"RUNNING $command\\n\\n\"); \n' ...
    '\t' 'close(LOGFILE); \n' ...
    '\t' 'flock LOGFILE, 8; \n' ... 
    '\n' ...
    '\t' '#Run the actual command \n' ...
    '\t' 'system($command); \n' ...
    '}' ...
    ];
else 
    footer = [ ...
    '\n' ...
    '\t' '#No command logging - change this in conf.surprise.logCommand \n' ... 
    '\n' ...
    '\t' '#Run the actual command \n' ...
    '\t' 'system($command); \n' ...
    '}' ...
    ];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function conf = print_body_parts(conf,type)

conf.body = struct('Description','Holds strings used in creating process RSVP body');

conf.body.command   = ['\t' '$command = \"' conf.binary ' \". \n'];

if isfield(conf,'useSaliencyOnly') && strcmp(conf.useSaliencyOnly,'yes')
%    conf.body.stdpre1   = '\t\"--movie --ior-type=None --agm-type=Std \". \n';	
%    conf.body.stdpre2   = '\t\"--nouse-random --display-foa --ior-type=None \". \n';
%    conf.body.stdpre3   = '\t\"--sm-type=Trivial --wta-type=None \". \n';% 
    conf.body.stdpre1   = '\t\"--movie --agm-type=Std \". \n';	
    conf.body.stdpre2   = '\t\"--nouse-random --display-foa \". \n';
    conf.body.stdpre3   = '\t\" \". \n';
else
    conf.body.stdpre1   = '\t\"--movie --ior-type=None --agm-type=Std \". \n';	
    conf.body.stdpre2   = '\t\"--nouse-random --display-foa --ior-type=None \". \n';
    conf.body.stdpre3   = '\t\"--sm-type=Trivial --wta-type=None \". \n';
end

if isfield(conf,'inputStr')
    conf.body.input     = conf.inputStr;
else
    if(strcmp(type,'normal')) || (strcmp(type,'mask')) || (strcmp(type,'hard_mask')) || (strcmp(type,'not-hard_mask')) || (strcmp(type,'trans')) || ...
      (strcmp(type,'trans_mask'))
        conf.body.input     = '\t\"--in=$d/$d_${fbase}_#.png --input-frames=0-MAX\\@$fdur \". \n';
    elseif(strcmp(type,'hard'))
        conf.body.input     = '\t\"--in=$d/$filep.png --input-frames=0-MAX\\@$fdur \". \n'; 
    elseif(strcmp(type,'AB'))
        conf.body.input     = '\t\"--in=$d/stim_${fp1}_${fp2}_${fp3}_#.png --input-frames=0-MAX\\@$fdur \". \n';
    else
        error('Need to give a run type or include it as an option');
    end
end
conf.body.outmpeg   = '\t\"--out=mpeg:$d/stim- \". \n';
conf.body.outframe  = '\t\"--output-frames=0-MAX\\@$fdur \". \n';
conf.body.outpng    = '\t\"--out=png:$d/stim \". \n';
conf.body.stats1    = '\t\"--sv-type=Stats --sv-stats-fname=$d/stats.txt --save-channel-stats \". \n';
conf.body.stats2    = '\t\"--save-channel-stats-name=$d/chan.txt \". \n';
conf.body.stats3    = '\t\"--save-channel-stats-tag=$d_${fbase} --save-stats-per-channel \". \n';

if isfield(conf,'getFrequencyData') && strcmp(conf.getFrequencyData,'yes')
    conf.body.stats4    = '\t\"--save-stats-per-channel-freq \". \n'; 
else
    conf.body.stats4    = ''; 
end

if((isfield(conf,'runMaskSet')        && strcmp(conf.runMaskSet,'yes'))         || ...
   (isfield(conf,'runHardMaskSet')    && strcmp(conf.runHardMaskSet,'yes'))     || ...
   (isfield(conf,'runNotHardMaskSet') && strcmp(conf.runNotHardMaskSet,'yes'))  || ...
   (isfield(conf,'runTransMaskSet')   && strcmp(conf.runTransMaskSet,'yes')))
    conf.body.stats4    = [conf.body.stats4 '\t\"--save-ag --ag-type=Std --savestats-ag --ag-statsfile=ag.stats.txt --ag-tframe=0 --ag-maskfile=$d/mask.png \". \n'];
elseif(strcmp(type,'AB'))
    conf.body.stats4    = [conf.body.stats4 '\t\"--save-ag --ag-type=Std \". \n']; 
else
    conf.body.stats4    = [conf.body.stats4 '\t\"--save-ag --ag-type=Std \". \n']; 
end

conf.body.redir     = '\t\"$redir\"; \n';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function conf = print_body_parts_legacy(conf,type)

if((isfield(conf,'runMaskSet')        && strcmp(conf.runMaskSet,'yes'))         || ...
   (isfield(conf,'runHardMaskSet')    && strcmp(conf.runHardMaskSet,'yes'))     || ...
   (isfield(conf,'runNotHardMaskSet') && strcmp(conf.runNotHardMaskSet,'yes'))  || ...   
   (isfield(conf,'runABSet')          && strcmp(conf.runABSet,'yes'))  || ...
   (isfield(conf,'runTransMaskSet')   && strcmp(conf.runTransMaskSet,'yes')))
    error('Legacy and Mask not supported in the same code base');
end

conf.body = struct('Description','Holds strings used in creating process RSVP body');

conf.body.command   = '\t$command = \"/lab/nerd-cam/saliency.code.archive/saliency.jul.2006/bin/ezvision \". \n';
conf.body.stdpre1   = '\t\"--movie --ior-type=None \". \n';	
conf.body.stdpre2   = '\t\"--nouse-random --display-foa --ior-type=None \". \n';
conf.body.stdpre3   = '\t\"--sm-type=Trivial --wta-type=None \". \n';
if isfield(conf,'inputStr')
    conf.body.input     = conf.inputStr;
else
    if(strcmp(type,'normal'))
        conf.body.input     = '\t\"--in=$d/${fbase}_#.png --input-frames=0-MAX\\@$fdur \". \n';
    elseif(strcmp(type,'hard'))
        conf.body.input     = '\t\"--in=$d/$filep.png --input-frames=0-MAX\\@$fdur \". \n'; 
    end
end
conf.body.outmpeg   = '\t\"--out=mpeg:$d/stim- \". \n';
conf.body.outframe  = '\t\"--output-frames=0-MAX\\@$fdur \". \n';
conf.body.outpng    = '\t\"--out=png:$d/stim --save-salmap \". \n';
conf.body.salfac    = '\t\"--salmap-factor=1e12 --agm-factor=1e12 \". \n';
conf.body.stats1    = '\t\"--sv-stats-fname=$d/stats.txt \". \n';
conf.body.stats2    = '\t\"--save-channel-stats \". \n';
conf.body.stats3    = '\t\"--save-channel-stats-name=$d/chan.txt \". \n';
conf.body.savchan   = '\t\"--save-channel-stats-tag=$d_${fbase} \". \n';
conf.body.redir     = '\t\"$redir\"; \n';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [body,conf] = print_body_standard(conf,type)

if isfield(conf,'useLegacy') && strcmp(conf.useLegacy,'yes')
    fprintf('<NOTE> Using Legacy Saliency Code\n');
    
    conf = print_body_parts_legacy(conf,type);
    
    if strcmp(conf.condString,'UCIO_legacy')
        body = [ ...
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"-X --surprise --vc-chans=CIO \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.salfac ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.savchan ...
        conf.body.redir ...
        ];

    elseif strcmp(conf.condString,'UHIO_legacy')
	body = [ ...
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"-X --surprise --vc-chans=HIO \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.salfac ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.savchan ...
        conf.body.redir ...
    ];

    elseif strcmp(conf.condString,'UHIOLTX_legacy')
	body = [ ...
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"-X --surprise --vc-chans=HIOLTX \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.salfac ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.savchan ...
        conf.body.redir ...
    ];

    elseif strcmp(conf.condString,'UHIOGKSE_legacy')
 	body = [ ...
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"-X --surprise --vc-chans=HIOGKSE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.salfac ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.savchan ...
        conf.body.redir ...
    ];
    else 
        error('Requested Legacy setup, but no legacy type found as %s',conf.condString);
    end
    
elseif isfield(conf,'useSaliencyOnly') && strcmp(conf.useSaliencyOnly,'yes')
    fprintf('<NOTE> Using Saliency ONLY\n')
    
    conf = print_body_parts(conf,type); 
            
    if strcmp(conf.condString,'UHIO_basic') || strcmp(conf.condString,'UHIO_opt') || strcmp(conf.condString,'UHIO_max')  
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--vc-chans=HIO \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
	elseif strcmp(conf.condString,'UHOLTXE_basic') || strcmp(conf.condString,'UHOLTXE_opt') || strcmp(conf.condString,'UHOLTXE_max')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--vc-chans=HOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
	elseif strcmp(conf.condString,'UQOLTXE_basic') || strcmp(conf.condString,'UQOLTXE_opt') || strcmp(conf.condString,'UQOLTXE_max')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--vc-chans=QOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UCIOLTXE_basic') || strcmp(conf.condString,'UCIOLTXE_opt') || strcmp(conf.condString,'UCIOLTXE_max')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--vc-chans=CIOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];   
    elseif strcmp(conf.condString,'UIOLTXE_basic') || strcmp(conf.condString,'UIOLTXE_opt') || strcmp(conf.condString,'UIOLTXE_max')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--vc-chans=IOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    else
        error('Requested Saliency ONLY setup, but no type found as %s',conf.condString);
    end
    
else
    fprintf('<NOTE> Using Standard Saliency Code\n');
    
    conf = print_body_parts(conf,type); 
    
    if strcmp(conf.condString,'UCIO_old')   
        body = [ ...    
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=C:0.8333I:0.4167O:1.6667F:0.4167M:1.6667 \". \n' ...
        '\t\"--display-map=SM --display-map-factor=50000 --vcx-normsurp=0 \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];

    elseif strcmp(conf.condString,'NATHAN_UCIO_basic') || strcmp(conf.condString,'NATHAN_UCIO_opt')
        body = [ ...       
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=CIO \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ... 
        conf.body.stats4 ...
        '\t\"--surprise-type=Nathan \". \n' ...	
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];

    elseif strcmp(conf.condString,'JointGG_UCIO_basic') || strcmp(conf.condString,'JointGG_UCIO_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=CIO \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=JointGG \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];  
    elseif strcmp(conf.condString,'ChiSq_UHIO_basic') || strcmp(conf.condString,'ChiSq_UHIO_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIO \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=ChiSquare \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];
    elseif strcmp(conf.condString,'ChiSq_UHOLTXE_basic') || strcmp(conf.condString,'ChiSq_UHOLTXE_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=ChiSquare \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];    
    elseif strcmp(conf.condString,'ChiSq_UHIOLTXE_basic') || strcmp(conf.condString,'ChiSq_UHIOLTXE_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=ChiSquare \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];
    %'\t\"--surprise --vc-chans=UHIO \". \n' ... 
    elseif strcmp(conf.condString,'JointGG_UHIO_basic') || strcmp(conf.condString,'JointGG_UHIO_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=H:0.877309234I:0.312778365O:1.566049997 \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=JointGG \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];
     
    elseif strcmp(conf.condString,'JointGG_UHOLTXE_basic') || strcmp(conf.condString,'JointGG_UHOLTXE_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=JointGG \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];
    
    elseif strcmp(conf.condString,'JointGG_UHIOLTXE_basic') || strcmp(conf.condString,'JointGG_UHIOLTXE_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=JointGG \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];
    
	elseif strcmp(conf.condString,'JointGG_UHIOE_basic') || strcmp(conf.condString,'JointGG_UHIOE_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=JointGG \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];    
    
    elseif strcmp(conf.condString,'JointGG_UHIOL_basic') || strcmp(conf.condString,'JointGG_UHIOL_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOL \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=JointGG \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];    
    
    elseif strcmp(conf.condString,'JointGG_UHIOT_basic') || strcmp(conf.condString,'JointGG_UHIOT_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=JointGG \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];
    
	elseif strcmp(conf.condString,'JointGG_UHIOX_basic') || strcmp(conf.condString,'JointGG_UHIOX_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=JointGG \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];
    
    elseif strcmp(conf.condString,'Gaussian_UHIO_basic')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIO \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=Gaussian \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];    
    
    elseif strcmp(conf.condString,'Gaussian_UHOLTXE_basic')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=Gaussian \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];    
    %'\t\"--surprise --vc-chans=UH:0.877309234I:0.312778365O:1.566049997 \". \n' ...
    elseif strcmp(conf.condString,'Outlier_UHIO_basic') || strcmp(conf.condString,'Outlier_UHIO_opt')    
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIO \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=Outlier \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];    
    
    elseif strcmp(conf.condString,'Outlier_UHOLTXE_basic') || strcmp(conf.condString,'Outlier_UHOLTXE_opt')    
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=Outlier \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];    
    
	elseif strcmp(conf.condString,'PoissonConst_UHIOE_basic') || strcmp(conf.condString,'PoissonConst_UHIOE_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=PoissonConst \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];    
    
    elseif strcmp(conf.condString,'PoissonConst_UHIOL_basic') || strcmp(conf.condString,'PoissonConst_UHIOL_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOL \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=PoissonConst \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];    
    
    elseif strcmp(conf.condString,'PoissonConst_UHIOT_basic') || strcmp(conf.condString,'PoissonConst_UHIOT_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOT \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=PoissonConst \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];
    
	elseif strcmp(conf.condString,'PoissonConst_UHIOX_basic') || strcmp(conf.condString,'PoissonConst_UHIOX_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOX \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=PoissonConst \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];	
    
    elseif strcmp(conf.condString,'PoissonConst_UHOLTXE_basic') || strcmp(conf.condString,'PoissonConst_UHOLTXE_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=PoissonConst \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];	
    
    elseif strcmp(conf.condString,'PoissonConst_UQOLTXE_basic') || strcmp(conf.condString,'PoissonConst_UQOLTXE_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=QOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=PoissonConst \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];	 
    
    elseif strcmp(conf.condString,'PoissonConst_UCIOTXE_basic') || strcmp(conf.condString,'PoissonConst_UCIOLTXE_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=CIOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=PoissonConst \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];
    
    elseif strcmp(conf.condString,'PoissonFloat_UHOLTXE_basic') || strcmp(conf.condString,'PoissonFloat_UHOLTXE_opt')     
        body = [ ...         
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        '\t\"--surprise-type=PoissonFloat \". \n' ...
        conf.RSVP_extra_args ...
        conf.body.redir ... 
        ];	
    
    %        '\t\"--surprise --vc-chans=UHIO \". \n' ...
    %        '\t\"--surprise --vc-chans=UH:0.8333I:0.4167O:1.6667 \". \n' ...
    %        '\t\"--surprise --vc-chans=UH:0.877309234I:0.312778365O:1.566049997 \". \n' ...
    %        '\t\"--surprise --vc-chans=UH:0.877309234I:0.312778365O:1.566049997 \". \n' ...
    elseif strcmp(conf.condString,'UHIO_basic') || strcmp(conf.condString,'UHIO_opt') || strcmp(conf.condString,'UHIO_max')  
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIO \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UHIO_train')  || strcmp(conf.condString,'UHIOLTWXE_train') || strcmp(conf.condString,'UHIOLTXE_train') || ...
           strcmp(conf.condString,'UHIOE_train') || strcmp(conf.condString,'UHIOL_train')     || strcmp(conf.condString,'UHIOT_train')    || ...
           strcmp(conf.condString,'UHIOX_train')
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        conf.trainStr ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
      elseif strcmp(conf.condString,'UHOLTWX_basic') || strcmp(conf.condString,'UHOLTWX_opt')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HOLTWX \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UHIOLTWX_basic') || strcmp(conf.condString,'UHIOLTWX_opt')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOLTWX \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UHIOLTWXE_basic') || strcmp(conf.condString,'UHIOLTWXE_opt')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOLTWXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UHIOLTXE_basic') || strcmp(conf.condString,'UHIOLTXE_opt') || strcmp(conf.condString,'UHIOLTXE_max')       
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
     elseif strcmp(conf.condString,'UHOLTXE_basic') || strcmp(conf.condString,'UHOLTXE_opt') || strcmp(conf.condString,'UHOLTXE_max')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
	elseif strcmp(conf.condString,'UQOLTXE_basic') || strcmp(conf.condString,'UQOLTXE_opt') || strcmp(conf.condString,'UQOLTXE_max')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=QOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UCIOLTXE_basic') || strcmp(conf.condString,'UCIOLTXE_opt') || strcmp(conf.condString,'UCIOLTXE_max')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=CIOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UIOLTXE_basic') || strcmp(conf.condString,'UIOLTXE_opt') || strcmp(conf.condString,'UIOLTXE_max')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=IOLTXE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
	elseif strcmp(conf.condString,'UHIOE_basic') || strcmp(conf.condString,'UHIOE_opt')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UHIOL_basic') || strcmp(conf.condString,'UHIOL_opt')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HIOL \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UHIOT_basic') || strcmp(conf.condString,'UHIOT_opt')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=UHIOT \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UHIOX_basic') || strcmp(conf.condString,'UHIOX_opt')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=UHIOX \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UHO_basic') || strcmp(conf.condString,'UHO_opt')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HO \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    elseif strcmp(conf.condString,'UHOE_basic') || strcmp(conf.condString,'UHOE_opt')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HOE \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UHOL_basic') || strcmp(conf.condString,'UHOL_opt')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HOL \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UHOT_basic') || strcmp(conf.condString,'UHOT_opt')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HOT \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UHOX_basic') || strcmp(conf.condString,'UHOX_opt')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HOX \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UHOW_basic') || strcmp(conf.condString,'UHOW_opt')        
        body = [ ...      
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=HOW \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    
    elseif strcmp(conf.condString,'UCIO_basic') || strcmp(conf.condString,'UCIO_opt')     
        body = [ ...   
        conf.body.command ...
        conf.body.stdpre1 ...
        '\t\"--surprise --vc-chans=CIO \". \n' ...
        conf.body.stdpre2 ...
        conf.body.stdpre3 ...
        conf.body.input ...
        conf.body.outframe ...
        conf.body.outpng ...
        conf.body.stats1 ...
        conf.body.stats2 ...
        conf.body.stats3 ...
        conf.body.stats4 ...
        conf.RSVP_extra_args ...
        conf.body.redir ...
        ];
    else
        error('Requested Non-Legacy setup, but no type found as %s',conf.condString);
    end
end 



