#!/usr/bin/perl

##########################################################################
## The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2002   ##
## by the University of Southern California (USC) and the iLab at USC.  ##
## See http://iLab.usc.edu for information about this project.          ##
##########################################################################
## Major portions of the iLab Neuromorphic Vision Toolkit are protected ##
## under the U.S. patent ``Computation of Intrinsic Perceptual Saliency ##
## in Visual Environments, and Applications'' by Christof Koch and      ##
## Laurent Itti, California Institute of Technology, 2001 (patent       ##
## pending; application number 09/912,225 filed July 23, 2001; see      ##
## http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     ##
##########################################################################
## This file is part of the iLab Neuromorphic Vision C++ Toolkit.       ##
##                                                                      ##
## The iLab Neuromorphic Vision C++ Toolkit is free software; you can   ##
## redistribute it and/or modify it under the terms of the GNU General  ##
## Public License as published by the Free Software Foundation; either  ##
## version 2 of the License, or (at your option) any later version.     ##
##                                                                      ##
## The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  ##
## that it will be useful, but WITHOUT ANY WARRANTY; without even the   ##
## implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ##
## PURPOSE.  See the GNU General Public License for more details.       ##
##                                                                      ##
## You should have received a copy of the GNU General Public License    ##
## along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   ##
## to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   ##
## Boston, MA 02111-1307 USA.                                           ##
##########################################################################
##
## Primary maintainer for this file: Laurent Itti <itti@usc.edu>
## $Id: beoload.sh 5534 2005-08-12 18:52:17Z itti $
##

# USAGE: psyframerate <file.psy> ... <file.psy>
#
# will display the average framerate for the overlay displays in the
# given .psy files (these files would be generated by, e.g.,
# psycho-movie or similar programs in saliency/src/AppPsycho/)

foreach $f (@ARGV) {
    open F, $f || die "Cannot open $f: ";

    $sum = 0.0; $n = 0;
    while(<F>) {
        chomp; ($t, $o) = split(/\s+/, $_, 2);

        if ($o eq 'displayYUVoverlay - frame 0') { $tref = $t; }
        elsif ($o =~ m/displayYUVoverlay - frame/)
        {
            my $tdiff = (utim($t) - utim($tref)) / 1000;
            if ($tdiff > 40.0)
            { print STDERR "Slow frame ignored: ${tdiff}ms at $t\n"; }
            else
            { $sum += $tdiff; $n ++; }

            $tref = $t;
        }
    }
    close F;
    if ($n == 0) { die "No YUV overlay displays in file $f\n"; }
    $avg = $sum / $n;
    print "$f $avg\n";
}


sub utim { # time
    my @x = split(/[:\.]/, $_[0]);
    return $x[0] * 3.6e9 + $x[1] * 6.0e7 + $x[2] * 1.0e6 +
        $x[3] * 1.0e3 + $x[4];
}
