/*!@file AppNeuro/featureVectorDistance.C program computes the distance between the 42 dimension feature vectors for two images  */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2005   //
// by the University of Southern California (USC) and the iLab at USC.  //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Farhan Baluch <fbaluch at usc dot edu>
// $HeadURL: svn://isvn.usc.edu/software/invt/trunk/saliency/src/AppNeuro/featureVectorDistance.C $
// $Id: featureVectorDistance.C 10794 2009-02-08 06:21:09Z itti $
//

#ifndef APPNEURO_FEATUREVECTORDISTANCE_C_DEFINED
#define APPNEURO_FEATUREVECTORDISTANCE_C_DEFINED

#include "Component/ModelManager.H"
#include "Image/Image.H"
#include "Image/Pixels.H"
#include "Image/ShapeOps.H" // for rescale()
#include "Image/FilterOps.H" //for correlation();
#include "Image/CutPaste.H"
#include "Neuro/getSaliency.H"
#include "Raster/Raster.H"
#include "Util/log.H"
#include "Psycho/EyeTrace.H"
#include "Image/MathOps.H"
#include "Util/StringConversions.H"
#include <fstream>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
//this code will take as input an image file and an eye file and computes NSS for all submaps generated by the model for the image.


int main(int argc, char** argv)
{
  ModelManager manager("test");

  nub::ref<GetSaliency> saliency(new GetSaliency(manager));
  manager.addSubComponent(saliency);
  if (manager.parseCommandLine(argc, argv, "<imageDirectory> <specDirectory> <outputDirectory>", 3, 3) == false)
    return -1;
  manager.start();

  std::string trgtImgFile, arrayImgFile, specFile, outTrgFile,
      outArrFile,outNameFile, corFile;
  size_t numSubMap;
  std::ofstream outputFile;


  outNameFile = manager.getExtraArg(2) + "mapNames.dat";
  outputFile.open(outNameFile.c_str());

  for(int i=1;i<=5;i++)
  {
      trgtImgFile  =  manager.getExtraArg(0) + convertToString(i) + "TARGET.png";
      arrayImgFile =  manager.getExtraArg(0) + convertToString(i) + "ARRAY.png";
      specFile     =  manager.getExtraArg(1) + convertToString(i) + "SPEC.datm";

      LINFO("computing maps for %s",arrayImgFile.c_str());

      const Image<PixRGB<byte> > trgtImg  = Raster::ReadRGB(trgtImgFile);
      const Image<PixRGB<byte> > arrayImg = Raster::ReadRGB(arrayImgFile);

      const int numSalientSpotsTr = saliency->compute(trgtImg, SimTime::SECS(0.1));
      LINFO("found %d salient spots", numSalientSpotsTr);
      const Image<float> salmap = saliency->getSalmap();
      const Image<float> resized_salmap = rescale(salmap, trgtImg.getDims());
      const std::vector<subMap> itsTrgtSubMaps = saliency->getSubMaps();

      const int numSalientSpotsArray = saliency->compute(arrayImg,SimTime::SECS(0.1));
      LINFO("found %d salient spots", numSalientSpotsArray);
      const std::vector<subMap> itsArraySubMaps = saliency->getSubMaps();

      numSubMap = itsTrgtSubMaps.size();

      //lets save the target and array submaps in a directory
      for (size_t j=0;j<numSubMap;j++)
      {
          outTrgFile = manager.getExtraArg(2) + convertToString(i) + "TRGsub"
              + convertToString(j) + ".png";
          LINFO("writing to file %s",outTrgFile.c_str());
          Raster::WriteFloat(itsTrgtSubMaps[j].itsSubMap,FLOAT_NORM_0_255,outTrgFile);

          outArrFile = manager.getExtraArg(2) + convertToString(i) + "ARRsub"
              + convertToString(j) + ".png";
          LINFO("writing to file %s",outArrFile.c_str());
          Raster::WriteFloat(itsArraySubMaps[j].itsSubMap,FLOAT_NORM_0_255,outArrFile);
          outputFile << itsArraySubMaps[j].itsSubMapName <<"\n";

          corFile = manager.getExtraArg(2) + convertToString(i) + "CorSub"
              + convertToString(j) + ".png";

          Image<float> temp = crop(itsArraySubMaps[j].itsSubMap,Point2D<int> (58,31),Dims(15,15));
          Raster::WriteFloat(correlation(itsArraySubMaps[j].itsSubMap,temp), FLOAT_NORM_0_255, corFile);

      }

  }

  outputFile.close();
  manager.stop();

  return 0;
}

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* mode: c++ */
/* indent-tabs-mode: nil */
/* End: */

#endif // APPNEURO_FEATUREVECTORDISTANCE_C_DEFINED
