/**
   \file  Robots/LoBot/config/LoLexYaccDefs.H
   \brief Just some declarations related to the INI-file lexer and parser
   modules built using lex and yacc.
*/

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2005   //
// by the University of Southern California (USC) and the iLab at USC.  //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Manu Viswanathan <mviswana at usc dot edu>
// $HeadURL: svn://isvn.usc.edu/software/invt/trunk/saliency/src/Robots/LoBot/config/LoLexYaccDefs.H $
// $Id: LoLexYaccDefs.H 11256 2009-05-30 01:46:10Z mviswana $
//

#ifndef LOBOT_LEX_YACC_DEFS_DOT_H
#define LOBOT_LEX_YACC_DEFS_DOT_H

//------------------------------ HEADERS --------------------------------

// Standard C headers
#include <stdio.h>

//------------------------------ DEFINES --------------------------------

// Enable parser debugging features
#define YYDEBUG 1

// Internal name for the "unnamed" top-level section of the INI file
#define LOCD_TOP_LEVEL "__ANONYMOUS_TOP_LEVEL__"

//------------------------------ GLOBALS --------------------------------

// Input stream used by the lexer
extern FILE* yyin ;

//----------------------------- FUNCTIONS -------------------------------

// The lexer routine generated by lex
int yylex(void) ;

// The parser routine generated by yacc
int yyparse(void) ;

// The error routine used by yacc
void yyerror(const char*) ;

// API provided by the parser module to allow lexer to pass tokens to it
void locd_ifp_set_current_identifier(const char*) ;
void locd_ifp_set_current_value(const char*) ;

//-----------------------------------------------------------------------

#endif // #ifndef LOBOT_LEX_YACC_DEFS_DOT_H

/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
