#!/usr/bin/perl -w

# $Id: testlong_contour_blackbox.pl 8853 2007-10-17 23:34:14Z rjpeters $

# This Perl script is a "black box" test driver. The idea is that we
# run the executable with different sets of command-line options, and
# compare the output files with reference files that are stored in the
# ref/ subdirectory. See blackbox.pm for implementation details.

use strict;

use blackbox;
use invt_config;

# here is the key to the test suite: the database of command-line
# options along with the expected output files for those option sets

my @tests =
    (
     {
	 name  => "contour-layer-static--1",
	 args  => ['../inputs/ezframe000000.pnm', 'out',
		   '--contour-cxn-type=BraunNorm',
		   '--contour-dynamics-type=Static',
		   '--contour-num-iterations=3',
		   '--maxnorm-type=Ignore',
		   '--save-contour-layer-output',
		   '--save-contour-layer-details',
		   '--save-contour-weights',
		   '--save-contour-filter-output',
		   '--save-contour-output',
		   '--out=pnm:#2#'],
	 files => ['out.avg00.pnm',
		   'out.filter.s-000.pnm',
		   'out.filter.s-100.pnm',
		   'out.filter.s-200.pnm',
		   'out.final00.pnm',
		   'out.gmod.s-0.i-0000.pnm',
		   'out.gmod.s-0.i-0100.pnm',
		   'out.gmod.s-0.i-0200.pnm',
		   'out.gmod.s-1.i-0000.pnm',
		   'out.gmod.s-1.i-0100.pnm',
		   'out.gmod.s-1.i-0200.pnm',
		   'out.gmod.s-2.i-0000.pnm',
		   'out.gmod.s-2.i-0100.pnm',
		   'out.gmod.s-2.i-0200.pnm',
		   'out.max00.pnm',
		   'out.output-s00.pnm',
		   'out.output00.pnm',
		   'out.potential.s-0.i-0000.pnm',
		   'out.potential.s-0.i-0100.pnm',
		   'out.potential.s-0.i-0200.pnm',
		   'out.potential.s-1.i-0000.pnm',
		   'out.potential.s-1.i-0100.pnm',
		   'out.potential.s-1.i-0200.pnm',
		   'out.potential.s-2.i-0000.pnm',
		   'out.potential.s-2.i-0100.pnm',
		   'out.potential.s-2.i-0200.pnm',
		   'out.weights00.pnm'],
     },

     {
	 name  => "contour-layer-static--2",
	 args  => ['../inputs/ezframe000000.pnm', 'out',
		   '--contour-cxn-type=Cinnic',
		   '--contour-dynamics-type=Static',
		   '--contour-num-iterations=3',
		   '--maxnorm-type=Ignore',
		   '--save-contour-layer-output',
		   '--save-contour-layer-details',
		   '--save-contour-weights',
		   '--save-contour-filter-output',
		   '--save-contour-output',
		   '--levelspec=2,4,3,4,1',
		   '--out=pnm:#2#'],
	 files => ['out.avg00.pnm',
		   'out.filter.s-000.pnm',
		   'out.filter.s-100.pnm',
		   'out.filter.s-200.pnm',
		   'out.final00.pnm',
		   'out.gmod.s-0.i-0000.pnm',
		   'out.gmod.s-0.i-0100.pnm',
		   'out.gmod.s-0.i-0200.pnm',
		   'out.gmod.s-1.i-0000.pnm',
		   'out.gmod.s-1.i-0100.pnm',
		   'out.gmod.s-1.i-0200.pnm',
		   'out.gmod.s-2.i-0000.pnm',
		   'out.gmod.s-2.i-0100.pnm',
		   'out.gmod.s-2.i-0200.pnm',
		   'out.max00.pnm',
		   'out.output-s00.pnm',
		   'out.output00.pnm',
		   'out.potential.s-0.i-0000.pnm',
		   'out.potential.s-0.i-0100.pnm',
		   'out.potential.s-0.i-0200.pnm',
		   'out.potential.s-1.i-0000.pnm',
		   'out.potential.s-1.i-0100.pnm',
		   'out.potential.s-1.i-0200.pnm',
		   'out.potential.s-2.i-0000.pnm',
		   'out.potential.s-2.i-0100.pnm',
		   'out.potential.s-2.i-0200.pnm',
		   'out.weights00.pnm'],
     },

     {
	 # use a very low SPLIT_SIZE=50000 to force the program to run
	 # the algorithm in tiles on the input image
	 name  => "contour-layer-static--3",
	 args  => ['../inputs/oddsize.pnm', 'out',
		   '--contour-cxn-type=BraunNorm',
		   '--contour-dynamics-type=Static',
		   '--contour-num-iterations=3',
		   '--maxnorm-type=Ignore',
		   '--save-contour-layer-output',
		   '--save-contour-layer-details',
		   '--save-contour-weights',
		   '--save-contour-filter-output',
		   '--save-contour-output',
		   '--out=pnm:#2#',
		   '50000'],
	 files => ['out-sub0.avg00.pnm',
		   'out-sub1.avg00.pnm',
		   'out-sub2.avg00.pnm',
		   'out-sub3.avg00.pnm',
		   'out-sub0.final00.pnm',
		   'out-sub1.final00.pnm',
		   'out-sub2.final00.pnm',
		   'out-sub3.final00.pnm',
		   'out.output00.pnm'],
     },

     {
	 name  => "contour-layer-dynamic--5",
	 args  => ['../inputs/ezframe000000.pnm', 'out',
		   '--contour-cxn-type=BraunNorm',
		   '--contour-dynamics-type=Dynamic',
		   '--contour-layer-timestep=0.5',
		   '--contour-layer-filter-thresh=0.0001',
		   '--contour-num-iterations=5',
		   '--maxnorm-type=Ignore',
		   '--save-contour-layer-output',
		   '--save-contour-layer-details',
		   '--save-contour-weights',
		   '--save-contour-filter-output',
		   '--save-contour-output',
		   '--out=pnm:#2#'],
	 files => ['out.all_chg.s-0.i-0000.pnm',
		   'out.all_chg.s-0.i-0100.pnm',
		   'out.all_chg.s-0.i-0200.pnm',
		   'out.all_chg.s-0.i-0300.pnm',
		   'out.all_chg.s-0.i-0400.pnm',
		   'out.all_chg.s-1.i-0000.pnm',
		   'out.all_chg.s-1.i-0100.pnm',
		   'out.all_chg.s-1.i-0200.pnm',
		   'out.all_chg.s-1.i-0300.pnm',
		   'out.all_chg.s-1.i-0400.pnm',
		   'out.all_chg.s-2.i-0000.pnm',
		   'out.all_chg.s-2.i-0100.pnm',
		   'out.all_chg.s-2.i-0200.pnm',
		   'out.all_chg.s-2.i-0300.pnm',
		   'out.all_chg.s-2.i-0400.pnm',
		   'out.all_exc.s-0.i-0000.pnm',
		   'out.all_exc.s-0.i-0100.pnm',
		   'out.all_exc.s-0.i-0200.pnm',
		   'out.all_exc.s-0.i-0300.pnm',
		   'out.all_exc.s-0.i-0400.pnm',
		   'out.all_exc.s-1.i-0000.pnm',
		   'out.all_exc.s-1.i-0100.pnm',
		   'out.all_exc.s-1.i-0200.pnm',
		   'out.all_exc.s-1.i-0300.pnm',
		   'out.all_exc.s-1.i-0400.pnm',
		   'out.all_exc.s-2.i-0000.pnm',
		   'out.all_exc.s-2.i-0100.pnm',
		   'out.all_exc.s-2.i-0200.pnm',
		   'out.all_exc.s-2.i-0300.pnm',
		   'out.all_exc.s-2.i-0400.pnm',
		   'out.all_inh.s-0.i-0000.pnm',
		   'out.all_inh.s-0.i-0100.pnm',
		   'out.all_inh.s-0.i-0200.pnm',
		   'out.all_inh.s-0.i-0300.pnm',
		   'out.all_inh.s-0.i-0400.pnm',
		   'out.all_inh.s-1.i-0000.pnm',
		   'out.all_inh.s-1.i-0100.pnm',
		   'out.all_inh.s-1.i-0200.pnm',
		   'out.all_inh.s-1.i-0300.pnm',
		   'out.all_inh.s-1.i-0400.pnm',
		   'out.all_inh.s-2.i-0000.pnm',
		   'out.all_inh.s-2.i-0100.pnm',
		   'out.all_inh.s-2.i-0200.pnm',
		   'out.all_inh.s-2.i-0300.pnm',
		   'out.all_inh.s-2.i-0400.pnm',
		   'out.avg00.pnm',
		   'out.filter.s-000.pnm',
		   'out.filter.s-100.pnm',
		   'out.filter.s-200.pnm',
		   'out.final00.pnm',
		   'out.max00.pnm',
		   'out.output-s00.pnm',
		   'out.output00.pnm',
		   'out.potential.s-0.i-0000.pnm',
		   'out.potential.s-0.i-0100.pnm',
		   'out.potential.s-0.i-0200.pnm',
		   'out.potential.s-0.i-0300.pnm',
		   'out.potential.s-0.i-0400.pnm',
		   'out.potential.s-1.i-0000.pnm',
		   'out.potential.s-1.i-0100.pnm',
		   'out.potential.s-1.i-0200.pnm',
		   'out.potential.s-1.i-0300.pnm',
		   'out.potential.s-1.i-0400.pnm',
		   'out.potential.s-2.i-0000.pnm',
		   'out.potential.s-2.i-0100.pnm',
		   'out.potential.s-2.i-0200.pnm',
		   'out.potential.s-2.i-0300.pnm',
		   'out.potential.s-2.i-0400.pnm',
		   'out.weights00.pnm'],
     },

     {
	 name  => "contour-model27--1",
	 args  => ['../inputs/ezframe000000.pnm', 'out27',
		   '--contour-model-0027',
		   '--save-contour-output',
		   '--contour-norm-flags=4',
		   '--out=pnm:#2#'],
	 files => ['out27.avg00.pfm'],
     },

     {
	 name  => "contour-model32--1",
	 args  => ['../inputs/ezframe000000.pnm', 'out32',
		   '--contour-model-0032',
		   '--save-contour-output',
		   '--contour-norm-flags=4',
		   '--out=pnm:#2#'],
	 files => ['out32.avg00.pfm'],
     },

     );


# Run the black box tests; note that the default executable can be
# overridden from the command-line with "--executable"

blackbox::run("$invt_config::exec_prefix/bin/contour", @tests);
